"""
Created on 26 mars 2018

@author: Stephane Tonneau

"""

from __future__ import absolute_import
import os
import datetime
import pidclient.pid_entry as pid_entry
import pidclient.log_entry as log_entry
import pidclient.base_utils as base_utils
from pidclient.environment_factory import EnvironmentFactory

PID_CONFIG_PATH = "pidconfigpath"


class LoggingFactory():
    """Class that encapsulates logging methods"""

    logging_factory = None

    @classmethod
    def logger(cls, product_id, product_type, product_date):
        """return a new ProcessLog object for a factory initialized by default

        :param product_id: the identifier of the new generated product
        :type product_id: string
        :param product_type: the product type of the new generated product
        :type product_type: string
        :param product_date: the creation date of the new generated product
        :type product_date: string

        :returns: the object itself
        :rtype: ProcessLog
        """
        if cls.logging_factory is None:
            cls.logging_factory = LoggingFactory()
        return cls.logging_factory.get_logger(product_id, product_type, product_date)


    def __init__(self, **logger_info):
        """Initialisator of the class

        :param logger_info: specific parameters of the logger class
        :type logger_info: a dictionary
        """
        self.logging_methods = []
        self.system_information = {}
        configuration = None
        config_path = None
        if PID_CONFIG_PATH in os.environ:
            config_path = os.environ[PID_CONFIG_PATH]
            base_utils.Tracer.debug("LoggingFactory",
                                    "Environment config_path = "+ config_path)
        if logger_info is not None:
            if "classes" in logger_info:
                self.logging_methods = logger_info["classes"]
                base_utils.Tracer.debug("LoggingFactory",
                                        "classes = "+ str(self.logging_methods))
            if PID_CONFIG_PATH in logger_info:
                config_path = logger_info[PID_CONFIG_PATH]
                base_utils.Tracer.debug("LoggingFactory",
                                        "Parameter config_path = " + config_path)
            if "sysinfo" in logger_info:
                self.system_information = logger_info["sysinfo"]
                base_utils.Tracer.debug("LoggingFactory",
                                        "sysinfo information = "+ str(self.system_information))
        if config_path is not None:
            configuration = base_utils.read_config(config_path)
            base_utils.Tracer.debug("LoggingFactory",
                                    "Configuration file read : " + config_path)
        if not self.logging_methods:
            if "loggers" in logger_info:
                logger_name = logger_info["loggers"]
                base_utils.Tracer.debug("LoggingFactory",
                                        "Parameter logger_name = "+ str(logger_name))
            else:
                logger_name = base_utils.get_config(
                    configuration, "main", "loggers")
                base_utils.Tracer.debug("LoggingFactory",
                                        "Configuration file logger_name = "+ str(logger_name))
            if logger_name is None:
                logger_tab = ("console")
                base_utils.Tracer.debug("LoggingFactory",
                                        "Logging set to default")
            else:
                logger_tab = logger_name.replace(" ", "").split(",")
                base_utils.Tracer.debug("LoggingFactory",
                                        "logging set to "+str(logger_tab))
            parameters = {"configuration" : configuration}
            base_utils.Tracer.debug("LoggingFactory",
                                    "Parameters set to configuration "+str(configuration))
            parameters.update(logger_info)
            if "file" in logger_tab:
                self.use_file(**parameters)
                base_utils.Tracer.debug("LoggingFactory",
                                        "Using file log with parameters "+str(parameters))
            if "elasticsearch" in logger_tab:
                self.use_elastic(**parameters)
                base_utils.Tracer.debug("LoggingFactory",
                                        "Using elasticsearch log with parameters "+str(parameters))
            if "console" in logger_tab:
                self.use_console(**parameters)
                base_utils.Tracer.debug("LoggingFactory",
                                        "Using console log with parameters "+str(parameters))
            if "kafka" in logger_tab:
                self.use_kafka(**parameters)
                base_utils.Tracer.debug("LoggingFactory",
                                        "Using kafka log with parameters "+str(parameters))

    def use_file(self, **parameters):
        """declare the filesystem as logging system

        :param parameters: the general logging parameters
        :type parameters: a python dictionary

        :returns: the object itself
        :rtype: LoggingFactory
        """
        import pidclient.pid_logging.file_log as file_log
        self.logging_methods.append(file_log.FileLogging(**parameters))
        return self

    def use_elastic(self, **parameters):
        """declare elastic search as logging system

        :param parameters: the general logging parameters
        :type parameters: a python dictionary

        :returns: the object itself
        :rtype: LoggingFactory
        """
        import pidclient.pid_logging.elasticsearch_log as elasticsearch_log
        self.logging_methods.append(
            elasticsearch_log.ElasticsearchLogging(**parameters))
        return self

    def use_console(self, **parameters):
        """declare the console as logging system

        :param parameters: the general logging parameters
        :type parameters: a python dictionary

        :returns: the object itself
        :rtype: LoggingFactory
        """
        import pidclient.pid_logging.console_log as console_log
        self.logging_methods.append(
            console_log.ConsoleLogging(**parameters))
        return self

    def use_kafka(self, **parameters):
        """declare kafka as logging system

        :param parameters: the general logging parameters
        :type parameters: a python dictionary

        :returns: the object itself
        :rtype: LoggingFactory
        """
        import pidclient.pid_logging.kafka_log as kafka_log
        self.logging_methods.append(kafka_log.KafkaLogging(**parameters))
        return self

    def get_logger(self, product_id, product_type, product_date):
        """return a new ProcessLog object

        :param product_id: the identifier of the new generated product
        :type product_id: string
        :param product_type: the product type of the new generated product
        :type product_type: string
        :param product_date: the creation date of the new generated product
        :type product_date: string

        :returns: the object itself
        :rtype: ProcessLog
        """
        new_product_id = product_id if product_id is not None else "-"
        new_product_type = product_type if product_type is not None else "-"
        new_product_date = product_date if product_date is not None else datetime.datetime.now()
        entry = pid_entry.PidEntry(new_product_id, new_product_type, new_product_date)
        environment = EnvironmentFactory(**self.system_information).environment
        entry.configuration = environment.configuration
        entry._job_id = environment.job_id
        entry.job_type = environment.job_type
        entry.user = environment.user
        entry.host = environment.host
        entry.parent_job_id = environment.parent_id
        entry.start_time = datetime.datetime.now()
        entry.environment = environment.environment_type
        meta_information = pid_entry.Metainformation()
        meta_information.uid = environment.get_uid()
        return log_entry.ProcessLog(entry, meta_information, self.logging_methods)
